package openngvfs

import (
	"github.com/spf13/afero"
	"os"
	"time"
)

var _ afero.Fs = (*OpenNagaeVFS)(nil)

func (n *OpenNagaeVFS) Create(name string) (afero.File, error) {
	f, err := n.root.Create(name)
	if err != nil {
		return nil, n.errorProcess(name, "create", err)
	}
	return f, nil
}

func (n *OpenNagaeVFS) Mkdir(name string, perm os.FileMode) error {
	err := n.root.Mkdir(name, perm)
	if err != nil {
		return n.errorProcess(name, "create", err)
	}
	return nil
}

func (n *OpenNagaeVFS) Name() string {
	return "nagae-vfs"
}

func (n *OpenNagaeVFS) MkdirAll(path string, perm os.FileMode) error {
	err := n.root.MkdirAll(path, perm)
	if err != nil {
		return n.errorProcess(path, "mkdirall", err)
	}
	return nil
}

func (n *OpenNagaeVFS) Open(name string) (afero.File, error) {
	f, err := n.root.Open(name)
	if err != nil {
		return nil, n.errorProcess(name, "open", err)
	}
	return f, nil
}

func (n *OpenNagaeVFS) OpenFile(name string, flag int, perm os.FileMode) (afero.File, error) {
	f, err := n.root.OpenFile(name, flag, perm)
	if err != nil {
		return nil, n.errorProcess(name, "openfile", err)
	}
	return f, nil
}

func (n *OpenNagaeVFS) Remove(name string) error {
	err := n.root.Remove(name)
	if err != nil {
		return n.errorProcess(name, "remove", err)
	}
	return nil
}

func (n *OpenNagaeVFS) RemoveAll(path string) error {
	err := n.root.RemoveAll(path)
	if err != nil {
		return n.errorProcess(path, "removeall", err)
	}
	return nil
}

func (n *OpenNagaeVFS) Rename(oldname, newname string) error {
	err := n.root.Rename(oldname, newname)
	if err != nil {
		return n.errorProcess(oldname, "rename", err)
	}
	return nil
}

func (n *OpenNagaeVFS) Stat(name string) (os.FileInfo, error) {
	info, err := n.root.Stat(name)
	if err != nil {
		return nil, n.errorProcess(name, "stat", err)
	}
	return info, nil
}

func (n *OpenNagaeVFS) Chmod(name string, mode os.FileMode) error {
	err := n.root.Chmod(name, mode)
	if err != nil {
		return n.errorProcess(name, "chmod", err)
	}
	return nil
}

func (n *OpenNagaeVFS) Chown(name string, uid, gid int) error {
	err := n.root.Chown(name, uid, gid)
	if err != nil {
		return n.errorProcess(name, "chown", err)
	}
	return nil
}

func (n *OpenNagaeVFS) Chtimes(name string, atime time.Time, mtime time.Time) error {
	err := n.root.Chtimes(name, atime, mtime)
	if err != nil {
		return n.errorProcess(name, "chtimes", err)
	}
	return nil
}
