package openngvfs

import (
	"fmt"
	"git.swzry.com/zry/GoHiedaLogger/hiedalog"
	"git.swzry.com/zry/io_writer_to_logger"
	"io"
)

var _ LogHandlerIntf = (*LoggerHiedaLogAdapter)(nil)

type LoggerHiedaLogAdapter struct {
	logger *hiedalog.HiedaLogger
}

func (e LoggerHiedaLogAdapter) FsInitWasm(name string) {
	e.logger.LogComplex("openngvfs", hiedalog.DLN_ERROR, map[string]string{
		"type": "will_init_by_wasm",
		"wasm": name,
	})
}

func (e LoggerHiedaLogAdapter) ChildFsError(mid int64, fstype, mountpoint, path, action string, rawErr error) {
	e.logger.LogComplex("openngvfs", hiedalog.DLN_ERROR, map[string]string{
		"type":    "child_fs_error",
		"path":    path,
		"fstype":  fstype,
		"action":  action,
		"mnt":     mountpoint,
		"mid":     fmt.Sprintf("%016X", mid),
		"raw_err": rawErr.Error(),
	})
}

func (e LoggerHiedaLogAdapter) NoAvailableMountPointForThisPath(path string, action string) {
	e.logger.LogComplex("openngvfs", hiedalog.DLN_ERROR, map[string]string{
		"type":   "no_mnt_point_error",
		"path":   path,
		"action": action,
	})
}

func (e LoggerHiedaLogAdapter) InitError(err error) {
	e.logger.LogComplex("openngvfs", hiedalog.DLN_ERROR, map[string]string{
		"type":    "openngvfs_init_error",
		"raw_err": err.Error(),
	})
}

func (e LoggerHiedaLogAdapter) UnknownError(path string, action string, err error) {
	e.logger.LogComplex("openngvfs", hiedalog.DLN_ERROR, map[string]string{
		"type":    "unknown_error",
		"path":    path,
		"action":  action,
		"raw_err": err.Error(),
	})
}

var _ io.WriteCloser = (*IOWriterAdapter)(nil)

type IOWriterAdapter struct {
	logger *hiedalog.HiedaLogger
	iowr   *io_writer_to_logger.WriterLogWrapper
	module string
	level  string
}

func (a IOWriterAdapter) Write(p []byte) (n int, err error) {
	return a.iowr.Write(p)
}

func (a IOWriterAdapter) Close() error {
	return a.iowr.Close()
}

func (a IOWriterAdapter) Run() {
	a.iowr.Run()
}

func (a IOWriterAdapter) log(msg string) {
	a.logger.LogString(a.module, a.level, msg)
}

func NewIOWriterAdapter(logger *hiedalog.HiedaLogger, module string, level string) *IOWriterAdapter {
	a := &IOWriterAdapter{
		logger: logger,
		module: module,
		level:  level,
	}
	a.iowr = io_writer_to_logger.NewWriterLogWrapper(a.log)
	return a
}
